# flake8: noqa

# import apis into api package
from eazyrent.products.v1.api.bulk_api import BulkApi
from eazyrent.products.v1.api.photos_api import PhotosApi
from eazyrent.products.v1.api.product_api import ProductApi
